<?php

/**
 * Metrics class for PROV - contains functions used for counting metrics for changes 
 *
 * @author GEIST Research Group
 */

class ProvMetrics {

	/** Function listing all categories in Doki except playground and wiki */
	private function list_all_categories() {
		$pathToPagesDir = DOKU_INC."/data/pages";
		$allClasses = scandir($pathToPagesDir);
		$classes = [];
		foreach($allClasses as $class) {
			if($class != "." && $class != ".." && $class != "wiki" && $class != "playground") {
				array_push($classes,$class);
			}
		}
		return $classes;
	}

	/** Function counting metrics called Attribute Richness */
	public function attributeRichness() {
		include_once(DOKU_INC."/lib/plugins/loki/utl/loki_utl_special.php");
		$utls = new LokiUtlSpecial;
		$attributes = $utls->list_attributes();
		$categories = $utls->list_categories();
		$ar = count($attributes)/count($categories);
		
		return $ar;
	}

	/** Function counting metrics called Class Richness - wrong... namespaces are not classes */
	public function classRichness() {
		include_once(DOKU_INC."/lib/plugins/loki/utl/loki_utl_special.php");
		$utls = new LokiUtlSpecial;
		$usedCategories = $utls->list_categories();
		$allCategories = $this->list_all_categories();

		$cr = (count($usedCategories)/count($allCategories))*100;

		return $cr;
	}

	/** Function counting metrics called Average Population */
	public function averagePopulation() {
		include_once(DOKU_INC."/lib/plugins/loki/utl/loki_utl_special.php");
		$utls = new LokiUtlSpecial;
		$categories = $utls->list_categories();
		$instanceNumber = 0;
		foreach($categories as $category){
			$instance = $utls->list_category_uses($category);
			$instanceNumber += count($instance);
		}
		$ap = $instanceNumber/count($categories);

		return $ap;
	}

	/** Function counting metrics called Size Of Vocabulary */
	public function sov() {
		include_once(DOKU_INC."/lib/plugins/loki/utl/loki_utl_special.php");
		$utls = new LokiUtlSpecial;
		$attributes = $utls->list_attributes();
		$categories = $utls->list_categories();
		$instanceNumber = 0;
		foreach($categories as $category){
			$instance = $utls->list_category_uses($category);
			$instanceNumber += count($instance);
		}
		$sov = count($attributes)+$instanceNumber;
		
		return $sov;
	}

	/** Function counting metrics called Edge Node Ratio */
	public function enr() {
		include_once(DOKU_INC."/lib/plugins/loki/utl/loki_utl_special.php");
		$utls = new LokiUtlSpecial;
		$relations = $utls->list_relations();
		$attributes = $utls->list_attributes();
		$categories = $utls->list_categories();
		$page_with_3 = [];
		$amount_3 = 0;
		$amout_attr_3 = 0;
		foreach($categories as $category) {
			$instance = $utls->list_category_uses($category);
			foreach($instance as $ins){
				$amount_3 = 0;
				$amout_attr_3 = 0;
				foreach($attributes as $attribute){
					$attr_uses = $utls->list_attribute_uses($attribute);
					$amount_3 += count($attr_uses);
					$amout_attr_3 += count($attr_uses);
					foreach($attr_uses as $value){
						if($value[0] == $ins){
							$page_with_3[$value[0]] = 1;							
						}
					}
				}
				foreach($relations as $relation) {
					$rel_uses = $utls->list_relation_uses($relation);
					$amount_3 += count($rel_uses);
					foreach($rel_uses as $value){
						if($value[0] == $ins){
							$page_with_3[$value[0]] = 1;						
						}
					}
				}
			}
		}
		$result = $amount_3/(count($page_with_3)+$amount_attr_3);
		return $result;
	}

}
?>
